# -*- coding: utf-8 -*-
# Copyright 2011 Thamini S.à.R.L    This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from osv import osv
from osv import fields
from tools.translate import _
import decimal_precision as dp

class insurance_contract_type_horse(osv.osv):
    _inherit = 'insurance.contract.type'
    _columns = {
        'horse_value_required': fields.boolean('Horse Value Required',
                    help='When check, the horse value field will be required on contract and also display in the contract report'),
    }
insurance_contract_type_horse()

class insurance_contract_horse(osv.osv):
    _inherit = 'insurance.contract'

    def _get_veterinary_certificate_status(self, cr, user, ids, fname, args, context=None):
        result = dict.fromkeys(ids, False)
        model_data_proxy = self.pool.get('ir.model.data')
        expert_report_proxy = self.pool.get('insurance.expert.report')

        _m, veterinary_certificate_type = model_data_proxy.get_object_reference(cr, user, 'horse_insurance', 'type_expert_report_veterinary_certificate')
        valid_certificates_ids = expert_report_proxy.search(cr, user, [
                                        ('type_id','=',veterinary_certificate_type),
                                        ('contract_id','in',ids),
                                        ('state','=','validated')
                                    ], context=context)

        for certificate in expert_report_proxy.browse(cr, user, valid_certificates_ids, context=context):
            result[certificate.contract_id.id] = True
        return result

    _columns = {
        'horse_id': fields.many2one('horse.horse', 'Horse', required=True),
        'horse_value': fields.float('Horse Value', digits_compute=dp.get_precision('Amount')),
        'horse_contact_ids': fields.many2many('horse.function', 'insurance_contract_horse_contacts_rel',
                                                'contract_id', 'horse_contact_id', 'Horse Contacts'),
        'veterinary_certificate_ok': fields.function(_get_veterinary_certificate_status, type='boolean', string='Veterinary Certificate OK?', method=True),
        'horse_value_required': fields.related('type_id', 'horse_value_required', type='boolean', string='Horse Value Required', readonly=True),
    }

    def onchange_horse_id(self, cr, uid, ids, horse_id, context=None):
        """
        Return some warning if the choosen horse has a preceding contract in refused state

        :param cr: database cursor
        :param uid: current user id
        :param ids: the current resource or list of resource the user is working on
        :param horse_id: the new horse id choosen by the user
        :param context: the context of the current request
        """
        if not horse_id:
            # force policyholder fields to be empty
            return {
                'value': {
                    'policyholder_id': False,
                    'policyholder_address_id': False,
                }
            }
        ocv = {'value': {}}
        # search if an owner (state == current) is present inside the horses functions
        # TODO: we need to improve this algorithm!
        horse_function_proxy = self.pool.get('horse.function')
        horse_function_ids = horse_function_proxy.search(cr, uid, [('horse_id','=',horse_id),('state','=','current'),('kind','=','owner')], context=context)
        if horse_function_ids:
            function = horse_function_proxy.browse(cr, uid, horse_function_ids[0], context=context)
            ocv['value'].update({
                'policyholder_id': function.partner_id.id,
                'policyholder_address_id': function.address_id.id,
            })

        # search for preceding contract in refused state
        # here we need to force admin uid because for some profil (ex: Insurance / Broker),
        # they won't have any read access to the preceding contract (ir.rule)
        prec_refused_contract_ids = self.search(cr, 1, [('horse_id', '=',horse_id),('state','=','refused')], order='refusal_date DESC')
        if prec_refused_contract_ids:
            prec_contract = self.read(cr, 1, prec_refused_contract_ids[0], context=context)
            ocv['warning'] = {
                'title': _('This horse has been previously refused'),
                'message': _('Previous refusal reason: %s') % (prec_contract['refusal_reason']),
            }
        else:
            claim_proxy = self.pool.get('insurance.claim')
            claim_associated_ids = claim_proxy.search(cr, 1, [('horse_id','=',horse_id)])
            if claim_associated_ids:
                ocv['warning'] = {
                    'title': _('Associated Claim Detected'),
                    'message': _('There is one or more claim declared for this horse'),
                }
        return ocv

    def onchange_horse_value(self, cr, uid, ids, horse_value, context=None):
        """
        Update insured amount when changing horse value

        :param cr: database cursor
        :param uid: current user id
        :param ids: the current resource or list of resource the user is working on
        :param horse_id: the new horse id choosen by the user
        :param context: the context of the current request
        """
        ocv = {'value': {'insured_amount': horse_value}}
        return ocv

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=100):
        """
        Search for records and their display names according to a search domain.
        """
        if args is None:
            args = []
        args.insert(0, ('horse_id', operator, name))
        if len(args) > 1:
            args.insert(0, '|')
        return super(insurance_contract_horse, self).name_search(cr, user, name, args, operator, context, limit)

insurance_contract_horse()

class insurance_claim_horse(osv.osv):
    _inherit = 'insurance.claim'
    _columns = {
        'horse_id': fields.related('contract_id', 'horse_id', type='many2one', relation='horse.horse', string='Horse', readonly=True),
        'policyholder_id': fields.related('contract_id', 'policyholder_id', type='many2one', relation='res.partner', string='Policyholder', readonly=True),
    }
insurance_claim_horse()

class insurance_expert_report_horse(osv.osv):
    _inherit = 'insurance.expert.report'
    _columns = {
        'horse_id': fields.related('contract_id', 'horse_id', type='many2one', relation='horse.horse', string='Horse', readonly=True),
    }
insurance_expert_report_horse()
